#!/usr/bin/env python
# -*- coding:utf-8 -*-


##############################################################################
## license :
##============================================================================
##
## File :        pg9530control.py
##
## Project :     TANGO Device Server
##
## This file is part of Tango device class.
##
## Tango is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Tango is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Tango.  If not, see <http://www.gnu.org/licenses/>.
##
##
## $Author :      sci.comp$
##
## $Revision :    $
##
## $Date :        $
##
## $HeadUrl :     $
##============================================================================
##            This file is generated by POGO
##    (Program Obviously used to Generate tango Object)
##
##        (c) - Software Engineering Group - ESRF
##############################################################################

"""This is a utilty device for controlling  the Quantum pg9530 via Tango attributes."""

__all__ = ["pg9530control", "pg9530controlClass", "main"]

__docformat__ = 'restructuredtext'

import PyTango
import sys
import time
# Add additional import
#----- PROTECTED REGION ID(pg9530control.additionnal_import) ENABLED START -----#

#----- PROTECTED REGION END -----#      //      pg9530control.additionnal_import

## Device States Description
## ON : The pulse is being generated.
## OFF : The pulse is not being generated.
## FAULT : The server pg9530 is not answering

class pg9530control (PyTango.Device_4Impl):

    #--------- Add you global variables here --------------------------
    #----- PROTECTED REGION ID(pg9530control.global_variables) ENABLED START -----#
    def initialize(self):
        self.flush();

        # Turn on echo (mandatory)
        #self.SendCommand(":SYST:COMM:SER:ECH 0")

        # Turn off display updates (mandatory)
        self.SendCommand(":DISP:MOD OFF")

        # Lock frontpanel (mandatory)
        self.SendCommand(":SYST:KLOC 0")

        # Turn off system beep (optional)
        self.SendCommand(":SYSTEM:BEEP:STAT OFF")

        # Set display bright (optional)
        self.SendCommand(":DISP:BRIG 0")

        # Turn on autorun (optional)
        #self.SendCommand(":SYST:AUT 1")


        # Send user's configuration to the device
        for cmd in self.Commands_Series_Text:
            if len(cmd) > 0:
                self.SendCommand(cmd)
        return

    def flush(self):
        if (self.devserial != None):
            #flush input and output
            self.devserial.DevSerFlush(2)
        return

    def send_query(self, cmd):
        ret = self.write_read(cmd);
        return ret;

    def write_read(self, cmd):
        if (self.devserial is None):
            return ""
        cmd += "\r\n"
        #self.flush()
        self.devserial.DevSerWriteString(cmd)
        time.sleep(0.1)
        tmp = ''
        reply = ''
        while len(tmp) == 0:
            tmp = self.devserial.DevSerReadRaw()
            reply += tmp
        '''
        #   Necessary only when ECHO is enabled
        unsigned int i;
        for (i = 0; i < reply.length(); i++)
                if (*(char *) (reply.c_str()+i) == 0xD)
                        break;
        strcpy(output,reply.c_str()+i+1);
        '''
        return reply.split("\r\n")[0]

    #----- PROTECTED REGION END -----#  //      pg9530control.global_variables

    def __init__(self,cl, name):
        PyTango.Device_4Impl.__init__(self,cl,name)
        self.debug_stream("In __init__()")
        pg9530control.init_device(self)
        #----- PROTECTED REGION ID(pg9530control.__init__) ENABLED START -----#

        #----- PROTECTED REGION END -----#      //      pg9530control.__init__

    def delete_device(self):
        self.debug_stream("In delete_device()")
        #----- PROTECTED REGION ID(pg9530control.delete_device) ENABLED START -----#
        # Set display bright
        self.SendCommand(":DISP:BRIG 4")

        # Unlock frontpanel
        self.SendCommand(":SYST:KLOC 0")

        # Turn on display updates
        self.SendCommand(":DISP:MOD ON")

        if self.devserial != None:
            del self.devserial
            self.devserial = None
        #----- PROTECTED REGION END -----#      //      pg9530control.delete_device

    def init_device(self):
        self.debug_stream("In init_device()")
        self.get_device_properties(self.get_device_class())
        self.attr_Period_read = 0.0
        self.attr_ChAWidth_read = 0.0
        self.attr_ChBWidth_read = 0.0
        self.attr_ChCWidth_read = 0.0
        self.attr_ChADelay_read = 0.0
        self.attr_ChBDelay_read = 0.0
        self.attr_ChCDelay_read = 0.0
        self.attr_ChASync_read = 0
        self.attr_ChBSync_read = 0
        self.attr_ChCSync_read = 0
        self.attr_Run_read = False
        #----- PROTECTED REGION ID(pg9530control.init_device) ENABLED START -----#
        try:
            self.devserial = PyTango.DeviceProxy(self.Serialdevice)
        except:
            self.devserial = None
        #
        self.set_state(PyTango.DevState.ON)
        #
        self.sync_modes = ["T0","CHA","CHB","CHC"]
        self.cha_sync = 0
        self.chb_sync = 0
        self.chc_sync = 0
        self.CheckState()
        #----- PROTECTED REGION END -----#      //      pg9530control.init_device

    def always_executed_hook(self):
        self.debug_stream("In always_excuted_hook()")
        #----- PROTECTED REGION ID(pg9530control.always_executed_hook) ENABLED START -----#

        #----- PROTECTED REGION END -----#      //      pg9530control.always_executed_hook

    #-----------------------------------------------------------------------------
    #    pg9530control read/write attribute methods
    #-----------------------------------------------------------------------------

    def read_Period(self, attr):
        self.debug_stream("In read_Period()")
        #----- PROTECTED REGION ID(pg9530control.Period_read) ENABLED START -----#
        attr.set_value(self.period)
        #----- PROTECTED REGION END -----#      //      pg9530control.Period_read

    def write_Period(self, attr):
        self.debug_stream("In write_Period()")
        data=attr.get_write_value()
        #----- PROTECTED REGION ID(pg9530control.Period_write) ENABLED START -----#
        self.period = data
        self.SetPeriod(self.period)
        #----- PROTECTED REGION END -----#      //      pg9530control.Period_write

    def read_ChAWidth(self, attr):
        self.debug_stream("In read_ChAWidth()")
        #----- PROTECTED REGION ID(pg9530control.ChAWidth_read) ENABLED START -----#
        attr.set_value(self.cha_width)
        #----- PROTECTED REGION END -----#      //      pg9530control.ChAWidth_read

    def write_ChAWidth(self, attr):
        self.debug_stream("In write_ChAWidth()")
        data=attr.get_write_value()
        #----- PROTECTED REGION ID(pg9530control.ChAWidth_write) ENABLED START -----#
        self.cha_width = data
        self.SetWidth([1,self.cha_width/1000.0])
        #----- PROTECTED REGION END -----#      //      pg9530control.ChAWidth_write

    def read_ChBWidth(self, attr):
        self.debug_stream("In read_ChBWidth()")
        #----- PROTECTED REGION ID(pg9530control.ChBWidth_read) ENABLED START -----#
        attr.set_value(self.chb_width)
        #----- PROTECTED REGION END -----#      //      pg9530control.ChBWidth_read

    def write_ChBWidth(self, attr):
        self.debug_stream("In write_ChBWidth()")
        data=attr.get_write_value()
        #----- PROTECTED REGION ID(pg9530control.ChBWidth_write) ENABLED START -----#
        self.chb_width = data
        self.SetWidth([2,self.chb_width / 1000.0])
        #----- PROTECTED REGION END -----#      //      pg9530control.ChBWidth_write

    def read_ChCWidth(self, attr):
        self.debug_stream("In read_ChCWidth()")
        #----- PROTECTED REGION ID(pg9530control.ChCWidth_read) ENABLED START -----#
        attr.set_value(self.chc_width)
        #----- PROTECTED REGION END -----#      //      pg9530control.ChCWidth_read

    def write_ChCWidth(self, attr):
        self.debug_stream("In write_ChCWidth()")
        data=attr.get_write_value()
        #----- PROTECTED REGION ID(pg9530control.ChCWidth_write) ENABLED START -----#
        self.chc_width = data
        self.SetWidth([3,self.chc_width / 1000.0])
        #----- PROTECTED REGION END -----#      //      pg9530control.ChCWidth_write

    def read_ChADelay(self, attr):
        self.debug_stream("In read_ChADelay()")
        #----- PROTECTED REGION ID(pg9530control.ChADelay_read) ENABLED START -----#
        attr.set_value(self.cha_delay)
        #----- PROTECTED REGION END -----#      //      pg9530control.ChADelay_read

    def write_ChADelay(self, attr):
        self.debug_stream("In write_ChADelay()")
        data=attr.get_write_value()
        #----- PROTECTED REGION ID(pg9530control.ChADelay_write) ENABLED START -----#
        self.cha_delay = data
        self.SetDelay([1,self.cha_delay/ 1000.0])
        #----- PROTECTED REGION END -----#      //      pg9530control.ChADelay_write

    def read_ChBDelay(self, attr):
        self.debug_stream("In read_ChBDelay()")
        #----- PROTECTED REGION ID(pg9530control.ChBDelay_read) ENABLED START -----#
        attr.set_value(self.chb_delay)
        #----- PROTECTED REGION END -----#      //      pg9530control.ChBDelay_read

    def write_ChBDelay(self, attr):
        self.debug_stream("In write_ChBDelay()")
        data=attr.get_write_value()
        #----- PROTECTED REGION ID(pg9530control.ChBDelay_write) ENABLED START -----#
        self.chb_delay = data
        self.SetDelay([2,self.chb_delay/ 1000.0])
        #----- PROTECTED REGION END -----#      //      pg9530control.ChBDelay_write

    def read_ChCDelay(self, attr):
        self.debug_stream("In read_ChCDelay()")
        #----- PROTECTED REGION ID(pg9530control.ChCDelay_read) ENABLED START -----#
        attr.set_value(self.chc_delay)
        #----- PROTECTED REGION END -----#      //      pg9530control.ChCDelay_read

    def write_ChCDelay(self, attr):
        self.debug_stream("In write_ChCDelay()")
        data=attr.get_write_value()
        #----- PROTECTED REGION ID(pg9530control.ChCDelay_write) ENABLED START -----#
        self.chc_delay = data
        self.SetDelay([3,self.chc_delay/ 1000.0])
        #----- PROTECTED REGION END -----#      //      pg9530control.ChCDelay_write

    def read_ChASync(self, attr):
        self.debug_stream("In read_ChASync()")
        #----- PROTECTED REGION ID(pg9530control.ChASync_read) ENABLED START -----#
        attr.set_value(self.cha_sync)

        #----- PROTECTED REGION END -----#      //      pg9530control.ChASync_read

    def write_ChASync(self, attr):
        self.debug_stream("In write_ChASync()")
        data=attr.get_write_value()
        #----- PROTECTED REGION ID(pg9530control.ChASync_write) ENABLED START -----#
        self.cha_sync = data
        self.cha_sync_name = self.sync_modes[self.cha_sync]
        cmd = ":PULSE1:SYNC "+self.cha_sync_name
        res = self.SendCommand(cmd)[:-2]
        if res != "ok":
            PyTango.Except.throw_exception("Failed","Command: "+ cmd + " returned = " + res ,"ChaSync")
        #----- PROTECTED REGION END -----#      //      pg9530control.ChASync_write

    def read_ChBSync(self, attr):
        self.debug_stream("In read_ChBSync()")
        #----- PROTECTED REGION ID(pg9530control.ChBSync_read) ENABLED START -----#
        attr.set_value(self.chb_sync)

        #----- PROTECTED REGION END -----#      //      pg9530control.ChBSync_read

    def write_ChBSync(self, attr):
        self.debug_stream("In write_ChBSync()")
        data=attr.get_write_value()
        #----- PROTECTED REGION ID(pg9530control.ChBSync_write) ENABLED START -----#
        self.chb_sync = data
        self.chb_sync_name = self.sync_modes[self.chb_sync]
        cmd = ":PULSE2:SYNC "+self.chb_sync_name
        res = self.SendCommand(cmd)[:-2]
        if res != "ok":
            PyTango.Except.throw_exception("Failed","Command: "+ cmd + " returned = " + res ,"ChbSync")
        #----- PROTECTED REGION END -----#      //      pg9530control.ChBSync_write

    def read_ChCSync(self, attr):
        self.debug_stream("In read_ChCSync()")
        #----- PROTECTED REGION ID(pg9530control.ChCSync_read) ENABLED START -----#
        attr.set_value(self.chc_sync)
        #----- PROTECTED REGION END -----#      //      pg9530control.ChCSync_read

    def write_ChCSync(self, attr):
        self.debug_stream("In write_ChCSync()")
        data=attr.get_write_value()
        #----- PROTECTED REGION ID(pg9530control.ChCSync_write) ENABLED START -----#
        self.chc_sync = data
        self.chc_sync_name = self.sync_modes[self.chc_sync]
        cmd = ":PULSE3:SYNC "+self.chc_sync_name
        res = self.SendCommand(cmd)[:-2]
        if res != "ok":
            PyTango.Except.throw_exception("Failed","Command: "+ cmd + " returned = " + res ,"ChcSync")
        #----- PROTECTED REGION END -----#      //      pg9530control.ChCSync_write

    def read_Run(self, attr):
        self.debug_stream("In read_Run()")
        #----- PROTECTED REGION ID(pg9530control.Run_read) ENABLED START -----#
        attr.set_value(self.to_state)
        #----- PROTECTED REGION END -----#      //      pg9530control.Run_read

    def write_Run(self, attr):
        self.debug_stream("In write_Run()")
        data=attr.get_write_value()
        #----- PROTECTED REGION ID(pg9530control.Run_write) ENABLED START -----#
        self.to_state = data
        if self.to_state:
            self.SendCommand(":PULSE0:STATE ON")
            self.set_state(PyTango.DevState.ON)
        else:
            self.SendCommand(":PULSE0:STATE OFF")
            self.set_state(PyTango.DevState.OFF)
        #----- PROTECTED REGION END -----#      //      pg9530control.Run_write



        #----- PROTECTED REGION ID(pg9530control.initialize_dynamic_attributes) ENABLED START -----#

        #----- PROTECTED REGION END -----#      //      pg9530control.initialize_dynamic_attributes

    def read_attr_hardware(self, data):
        self.debug_stream("In read_attr_hardware()")
        #----- PROTECTED REGION ID(pg9530control.read_attr_hardware) ENABLED START -----#

        #----- PROTECTED REGION END -----#      //      pg9530control.read_attr_hardware


    #-----------------------------------------------------------------------------
    #    pg9530control command methods
    #-----------------------------------------------------------------------------

    def CheckState(self):
        """ Check if the system is running or not.

        :param :
        :type: PyTango.DevVoid
        :return:
        :rtype: PyTango.DevVoid """
        self.debug_stream("In CheckState()")
        #----- PROTECTED REGION ID(pg9530control.CheckState) ENABLED START -----#
        import time
        t1 = time.time()
        self.to_state = self.IsEnabled(0)
        if self.to_state:
            self.set_state(PyTango.DevState.ON)
        else:
            self.set_state(PyTango.DevState.OFF)
        self.cha_width = float(self.SendCommand(':PULSE1:WIDT?')) * 1000 # CONVERTING TO ms
        self.cha_delay = float(self.SendCommand(':PULSE1:DELAY?')) * 1000 #CONVERTING TO ms
        self.cha_sync_name = self.SendCommand(':PULSE1:SYNC?')[-2]
        self.chb_width = float(self.SendCommand(':PULSE2:WIDT?')) * 1000 # CONVERTING TO ms
        self.chb_delay = float(self.SendCommand(':PULSE2:DELAY?')) * 1000 #CONVERTING TO ms
        self.chb_sync_name = self.SendCommand(':PULSE2:SYNC?')[-2]
        self.chc_width = float(self.SendCommand(':PULSE3:WIDT?')) * 1000 # CONVERTING TO ms
        self.chc_delay = float(self.SendCommand(':PULSE3:DELAY?')) * 1000 #CONVERTING TO ms
        self.chc_sync_name = self.SendCommand(':PULSE3:SYNC?')[-2]
        self.period = float(self.SendCommand(':PULSE0:PERIOD?'))
        for i in range(len(self.sync_modes)):
            if self.cha_sync_name == self.sync_modes[i]:
                self.cha_sync = i
            if self.chb_sync_name == self.sync_modes[i]:
                self.chb_sync = i
            if self.chc_sync_name == self.sync_modes[i]:
                self.chc_sync = i
        #----- PROTECTED REGION END -----#      //      pg9530control.CheckState

    def Arm(self):
        """

        :param :
        :type: PyTango.DevVoid
        :return:
        :rtype: PyTango.DevVoid """
        self.debug_stream("In Arm()")
        #----- PROTECTED REGION ID(pg9530control.Arm) ENABLED START -----#
        self.SendCommand("*ARM")
        #----- PROTECTED REGION END -----#      //      pg9530control.Arm

    def EnableChannel(self, argin):
        """ Enables the output pulse for selected channel. Channel 0 enables all pulses.

        :param argin: Number of channel
        :type: PyTango.DevUShort
        :return:
        :rtype: PyTango.DevVoid """
        self.debug_stream("In EnableChannel()")
        #----- PROTECTED REGION ID(pg9530control.EnableChannel) ENABLED START -----#
        command = ":PULSE" + str(argin) + ":STAT ON"
        self.SendCommand(command)
        #----- PROTECTED REGION END -----#      //      pg9530control.EnableChannel

    def DisableChannel(self, argin):
        """ Disables the output pulse for selected channel. Channel 0 disables all pulses.

        :param argin: Number of channel
        :type: PyTango.DevUShort
        :return:
        :rtype: PyTango.DevVoid """
        self.debug_stream("In DisableChannel()")
        #----- PROTECTED REGION ID(pg9530control.DisableChannel) ENABLED START -----#
        command = ":PULSE" + str(argin) + ":STAT OFF"
        self.SendCommand(command)
        #----- PROTECTED REGION END -----#      //      pg9530control.DisableChannel

    def SetDelay(self, argin):
        """ Sets a delay on the specified channel.
        Es. 1,0.03 sets a delay of 30 ms on channel 1

        :param argin: channel,delay
        :type: PyTango.DevVarDoubleArray
        :return:
        :rtype: PyTango.DevVoid """
        self.debug_stream("In SetDelay()")
        #----- PROTECTED REGION ID(pg9530control.SetDelay) ENABLED START -----#
        if len(argin) < 2:
            PyTango.Except.throw_exception("PARAMETERS ERROR","Incorrect number of parameters","SetDelay()")
        delay_val = "%0.10f" % argin[1]
        command = ":PULSE" + str(argin[0]) + ":DELAY " + delay_val
        self.SendCommand(command)
        #----- PROTECTED REGION END -----#      //      pg9530control.SetDelay

    def SetWidth(self, argin):
        """ Sets the width of the pulse no the specified channel.
        Es. 2,0.000025 sets a pulse width of 25 us on channel 2

        :param argin: Channel,width
        :type: PyTango.DevVarDoubleArray
        :return:
        :rtype: PyTango.DevVoid """
        self.debug_stream("In SetWidth()")
        #----- PROTECTED REGION ID(pg9530control.SetWidth) ENABLED START -----#
        if len(argin) < 2:
            PyTango.Except.throw_exception("PARAMETERS ERROR","Incorrect number of parameters","SetWidth()")
        val = "%0.10f" % argin[1]
        command = ":PULSE" + str(argin[0]) + ":WIDT " + val
        self.SendCommand(command)
        #----- PROTECTED REGION END -----#      //      pg9530control.SetWidth

    def SetPeriod(self, argin):
        """ Sets the T0 period

        :param argin: Value of T0
        :type: PyTango.DevDouble
        :return:
        :rtype: PyTango.DevVoid """
        self.debug_stream("In SetPeriod()")
        #----- PROTECTED REGION ID(pg9530control.SetPeriod) ENABLED START -----#
        val = "%0.10f" % argin
        command = ":PULSE0:PERIOD " + val
        self.SendCommand(command)
        #----- PROTECTED REGION END -----#      //      pg9530control.SetPeriod

    def SetMode(self, argin):
        """ Sets the T0 mode (normal, single, burst, dcycle). Enter a number from 1 to 4, same order.

        :param argin: The T0 mode
        :type: PyTango.DevUShort
        :return:
        :rtype: PyTango.DevVoid """
        self.debug_stream("In SetMode()")
        #----- PROTECTED REGION ID(pg9530control.SetMode) ENABLED START -----#
        command = ":PULSE0:MODE "
        if argin == 1:
            command += "NORMAL"
        elif argin == 2:
            command += "SINGLE"
        elif argin == 3:
            command += "BURST"
        elif argin == 4:
            command += "DCYCLE"
        else:
            PyTango.Except.throw_exception("PARAMETER ERROR","Parameter out of bounds","SetMode()")

        self.SendCommand(command)
        #----- PROTECTED REGION END -----#      //      pg9530control.SetMode

    def SetBurstCounter(self, argin):
        """ Sets the number of pulses to generate in Burst mode.

        :param argin: Number of pulses in Burst mode
        :type: PyTango.DevUShort
        :return:
        :rtype: PyTango.DevVoid """
        self.debug_stream("In SetBurstCounter()")
        #----- PROTECTED REGION ID(pg9530control.SetBurstCounter) ENABLED START -----#
        command = ":PULSE0:BCO " + str(argin)
        self.SendCommand(command)
        #----- PROTECTED REGION END -----#      //      pg9530control.SetBurstCounter

    def LockKeypad(self):
        """ Locks the keypad

        :param :
        :type: PyTango.DevVoid
        :return:
        :rtype: PyTango.DevVoid """
        self.debug_stream("In LockKeypad()")
        #----- PROTECTED REGION ID(pg9530control.LockKeypad) ENABLED START -----#
        command = ":SYST:KLOC 1"
        self.SendCommand(command)
        #----- PROTECTED REGION END -----#      //      pg9530control.LockKeypad

    def UnlockKeypad(self):
        """ Unlocks the keypad

        :param :
        :type: PyTango.DevVoid
        :return:
        :rtype: PyTango.DevVoid """
        self.debug_stream("In UnlockKeypad()")
        #----- PROTECTED REGION ID(pg9530control.UnlockKeypad) ENABLED START -----#
        command = ":SYST:KLOC 0"
        self.SendCommand(command)
        #----- PROTECTED REGION END -----#      //      pg9530control.UnlockKeypad

    def SendCommand(self, argin):
        """ Send any command you want and see what happens.

        :param argin: Command string
        :type: PyTango.DevString
        :return: Reply from pulse generator
        :rtype: PyTango.DevString """
        self.debug_stream("In SendCommand()")
        argout = ''
        #----- PROTECTED REGION ID(pg9530control.SendCommand) ENABLED START -----#
        try:
            argout = self.write_read(argin)
        except:
            PyTango.Except.throw_exception("send_command error","Serial communication error","SendCommand()")
        #----- PROTECTED REGION END -----#      //      pg9530control.SendCommand
        return argout

    def GetDelay(self, argin):
        """ Returns the delay of the channel in seconds

        :param argin: channel
        :type: PyTango.DevUShort
        :return: delay [s]
        :rtype: PyTango.DevDouble """
        self.debug_stream("In GetDelay()")
        argout = 0.0
        #----- PROTECTED REGION ID(pg9530control.GetDelay) ENABLED START -----#
        command = ":PULSE" + str(argin) + ":DELAY?"
        reply = self.send_query(command)
        argout = float(reply)
        #----- PROTECTED REGION END -----#      //      pg9530control.GetDelay
        return argout

    def IsEnabled(self, argin):
        """ Returns the state of the channel:
        0 - channel is disabled (no pulses)
        1 - channel is armed (can pulse)

        :param argin: channel
        :type: PyTango.DevUShort
        :return: enabled state
        :rtype: PyTango.DevBoolean """
        self.debug_stream("In IsEnabled()")
        argout = False
        #----- PROTECTED REGION ID(pg9530control.IsEnabled) ENABLED START -----#
        command = ":PULSE" + str(argin) + ":STATE?"
        reply = self.send_query(command)
        argout = int(reply)
        #----- PROTECTED REGION END -----#      //      pg9530control.IsEnabled
        return argout


class pg9530controlClass(PyTango.DeviceClass):
    #--------- Add you global class variables here --------------------------
    #----- PROTECTED REGION ID(pg9530control.global_class_variables) ENABLED START -----#

    #----- PROTECTED REGION END -----#  //      pg9530control.global_class_variables

    def dyn_attr(self, dev_list):
        """Invoked to create dynamic attributes for the given devices.
        Default implementation calls
        :meth:`pg9530control.initialize_dynamic_attributes` for each device

        :param dev_list: list of devices
        :type dev_list: :class:`PyTango.DeviceImpl`"""

        for dev in dev_list:
            try:
                dev.initialize_dynamic_attributes()
            except:
                import traceback
                dev.warn_stream("Failed to initialize dynamic attributes")
                dev.debug_stream("Details: " + traceback.format_exc())
        #----- PROTECTED REGION ID(pg9530control.dyn_attr) ENABLED START -----#

        #----- PROTECTED REGION END -----#      //      pg9530control.dyn_attr

    #    Class Properties
    class_property_list = {
        }


    #    Device Properties
    device_property_list = {
        'Serialdevice':
            [PyTango.DevString,
            "Tango name of the Serial device",
            ["diproi/instrumentation/Serial"] ],
        'Commands_Series_Text':
            [PyTango.DevString,
            "Series of commands which will be send together in one single step during Init phase or when user want trough Send_Commands_Series command.",
            [] ],
        }


    #    Command definitions
    cmd_list = {
        'CheckState':
            [[PyTango.DevVoid, "none"],
            [PyTango.DevVoid, "none"],
            {
                'Polling period': "10000",
                'Display level': PyTango.DispLevel.EXPERT,
            } ],
        'Arm':
            [[PyTango.DevVoid, "none"],
            [PyTango.DevVoid, "none"]],
        'EnableChannel':
            [[PyTango.DevUShort, "Number of channel"],
            [PyTango.DevVoid, "none"]],
        'DisableChannel':
            [[PyTango.DevUShort, "Number of channel"],
            [PyTango.DevVoid, "none"]],
        'SetDelay':
            [[PyTango.DevVarDoubleArray, "channel,delay"],
            [PyTango.DevVoid, "none"]],
        'SetWidth':
            [[PyTango.DevVarDoubleArray, "Channel,width"],
            [PyTango.DevVoid, "none"]],
        'SetPeriod':
            [[PyTango.DevDouble, "Value of T0"],
            [PyTango.DevVoid, "none"]],
        'SetMode':
            [[PyTango.DevUShort, "The T0 mode"],
            [PyTango.DevVoid, "none"]],
        'SetBurstCounter':
            [[PyTango.DevUShort, "Number of pulses in Burst mode"],
            [PyTango.DevVoid, "none"]],
        'LockKeypad':
            [[PyTango.DevVoid, "none"],
            [PyTango.DevVoid, "none"]],
        'UnlockKeypad':
            [[PyTango.DevVoid, "none"],
            [PyTango.DevVoid, "none"]],
        'SendCommand':
            [[PyTango.DevString, "Command string"],
            [PyTango.DevString, "Reply from pulse generator"]],
        'GetDelay':
            [[PyTango.DevUShort, "channel"],
            [PyTango.DevDouble, "delay [s]"]],
        'IsEnabled':
            [[PyTango.DevUShort, "channel"],
            [PyTango.DevBoolean, "enabled state"]],
        }


    #    Attribute definitions
    attr_list = {
        'Period':
            [[PyTango.DevDouble,
            PyTango.SCALAR,
            PyTango.READ_WRITE],
            {
                'label': "TO",
                'unit': "s",
                'display unit': "s",
                'description': "Period TO, PULSE0:PERIOD",
            } ],
        'ChAWidth':
            [[PyTango.DevDouble,
            PyTango.SCALAR,
            PyTango.READ_WRITE],
            {
                'label': "CHAWidth",
                'unit': "ms",
                'display unit': "ms",
                'description': "Pulse width of CHA; PULSE1:WIDT",
            } ],
        'ChBWidth':
            [[PyTango.DevDouble,
            PyTango.SCALAR,
            PyTango.READ_WRITE],
            {
                'label': "CHBWidth",
                'unit': "ms",
                'display unit': "ms",
                'description': "Pulse width of CHB; PULSE2:WIDT",
            } ],
        'ChCWidth':
            [[PyTango.DevDouble,
            PyTango.SCALAR,
            PyTango.READ_WRITE],
            {
                'label': "CHCWidth",
                'unit': "ms",
                'display unit': "ms",
                'description': "Pulse width of CHC; PULSE3:WIDT",
            } ],
        'ChADelay':
            [[PyTango.DevDouble,
            PyTango.SCALAR,
            PyTango.READ_WRITE],
            {
                'label': "CHADelay",
                'unit': "ms",
                'display unit': "ms",
                'description': "Delay of CHA; PULSE1:DELAY",
            } ],
        'ChBDelay':
            [[PyTango.DevDouble,
            PyTango.SCALAR,
            PyTango.READ_WRITE],
            {
                'label': "CHBDelay",
                'unit': "ms",
                'display unit': "ms",
                'description': "Delay of CHB; PULSE2:DELAY",
            } ],
        'ChCDelay':
            [[PyTango.DevDouble,
            PyTango.SCALAR,
            PyTango.READ_WRITE],
            {
                'label': "CHCDelay",
                'unit': "ms",
                'display unit': "ms",
                'description': "Delay of CHC; PULSE3:DELAY",
            } ],
        'ChASync':
            [[PyTango.DevLong,
            PyTango.SCALAR,
            PyTango.READ_WRITE],
            {
                'label': "ChASync",
                'description': "Chose the index of the CHASync Source: \n 0 --> TO\n 1 --> CHA\n 2 --> CHB\n 3 --> CHC\n\nPULSE1:SYNC (TO,CHA,CHB,CHC)",
            } ],
        'ChBSync':
            [[PyTango.DevLong,
            PyTango.SCALAR,
            PyTango.READ_WRITE],
            {
                'label': "ChBSync",
                'description': "Chose the index of the CHBSync Source: \n 0 --> TO\n 1 --> CHA\n 2 --> CHB\n 3 --> CHC\n\nPULSE2:SYNC (TO,CHA,CHB,CHC)",
            } ],
        'ChCSync':
            [[PyTango.DevLong,
            PyTango.SCALAR,
            PyTango.READ_WRITE],
            {
                'label': "ChCSync",
                'description': "Chose the index of the CHCSync Source: \n 0 --> TO\n 1 --> CHA\n 2 --> CHB\n 3 --> CHC\n\nPULSE3:SYNC (TO,CHA,CHB,CHC)",
            } ],
        'Run':
            [[PyTango.DevBoolean,
            PyTango.SCALAR,
            PyTango.READ_WRITE],
            {
                'label': "Start",
                'description': "Run/STOP the quantum producing the pulses as configured.\nPULSE0:STATE ON/ OFF",
            } ],
        }


def main():
    try:
        py = PyTango.Util(sys.argv)
        py.add_class(pg9530controlClass,pg9530control,'pg9530control')

        U = PyTango.Util.instance()
        U.server_init()
        U.server_run()

    except PyTango.DevFailed,e:
        print '-------> Received a DevFailed exception:',e
    except Exception,e:
        print '-------> An unforeseen exception occured....',e

if __name__ == '__main__':
    main()
